"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Source: https://github.com/madrobby/keymaster/blob/3b1f2afabf1569848dea8b697ac418f19b601a30/keymaster.js
 */
exports.KEY_MAP = {
    '⇧': 16, shift: 16,
    '⌥': 18, alt: 18, option: 18,
    '⌃': 17, ctrl: 17, control: 17,
    '⌘': 91, cmd: 91, command: 91, meta: 91,
    caps: 20, capslock: 20,
    backspace: 8, tab: 9, clear: 12,
    enter: 13, return: 13,
    esc: 27, escape: 27, space: 32,
    left: 37, up: 38, right: 39, down: 40,
    del: 46, delete: 46,
    home: 36, end: 35, pageup: 33, pagedown: 34,
    ',': 188, '.': 190, '/': 191,
    '`': 192, '-': 189, '=': 187,
    ';': 186, '\'': 222,
    '[': 219, ']': 221, '\\': 220
};
/**
 * Valid modifier keys.
 */
exports.MODIFIERS = [16, 17, 18, 91, 93, 224];
/**
 * Continue propagating the event to older listeners.
 */
exports.SHOULD_PROPAGATE = true;
/**
 * Stringify a keyboard event.
 */
function keyboardEventCombo(e) {
    var keys = [];
    if (e.shiftKey)
        keys.push(exports.KEY_MAP.shift);
    if (e.ctrlKey)
        keys.push(exports.KEY_MAP.ctrl);
    if (e.altKey)
        keys.push(exports.KEY_MAP.alt);
    if (e.metaKey)
        keys.push(exports.KEY_MAP.meta);
    if (exports.MODIFIERS.indexOf(e.which) === -1)
        keys.push(e.which);
    return keys.sort().join(' ');
}
exports.keyboardEventCombo = keyboardEventCombo;
/**
 * Map keys to string.
 */
function stringifyKey() {
    var keys = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        keys[_i] = arguments[_i];
    }
    return keys.map(function (key) {
        if (typeof key === 'number')
            return key;
        if (exports.KEY_MAP[key])
            return exports.KEY_MAP[key];
        if (key.length !== 1)
            throw new TypeError("Unknown key \"" + key + "\"");
        return key.toUpperCase().charCodeAt(0);
    }).sort().join(' ');
}
exports.stringifyKey = stringifyKey;
/**
 * Create a listener function from shortcuts.
 */
function createShortcuts(shortcuts, returnValue) {
    if (returnValue === void 0) { returnValue = exports.SHOULD_PROPAGATE; }
    return function (event, combo) {
        return shortcuts[combo] ? shortcuts[combo](event, combo) : returnValue;
    };
}
exports.createShortcuts = createShortcuts;
/**
 * Check if a keyboard event originated from an input.
 */
function isInputEvent(event) {
    var target = event.target;
    return (target.tagName === 'INPUT' ||
        target.tagName === 'SELECT' ||
        target.tagName === 'TEXTAREA' ||
        target.isContentEditable);
}
exports.isInputEvent = isInputEvent;
/**
 * Function to wrap listener by filtering input events.
 */
function filterInputEvent(listener) {
    return function (event, combo) {
        return isInputEvent(event) ? exports.SHOULD_PROPAGATE : listener(event, combo);
    };
}
exports.filterInputEvent = filterInputEvent;
/**
 * Keyboard manager library for mapping key events.
 */
var Keyboard = /** @class */ (function () {
    function Keyboard() {
        this.listeners = [];
    }
    Keyboard.prototype.addListener = function (callback) {
        return this.listeners.push(callback);
    };
    Keyboard.prototype.removeListener = function (callback) {
        var indexOf = this.listeners.indexOf(callback);
        if (indexOf > -1)
            this.listeners.splice(indexOf, 1);
        return indexOf;
    };
    Keyboard.prototype.getHandler = function () {
        var listener = this.getListener();
        return function (event) {
            return listener(event, keyboardEventCombo(event));
        };
    };
    Keyboard.prototype.getListener = function (returnValue) {
        var _this = this;
        if (returnValue === void 0) { returnValue = exports.SHOULD_PROPAGATE; }
        return function (event, combo) {
            var len = _this.listeners.length;
            while (len--) {
                if (_this.listeners[len](event, combo) !== exports.SHOULD_PROPAGATE)
                    return undefined;
            }
            return returnValue;
        };
    };
    return Keyboard;
}());
exports.Keyboard = Keyboard;
//# sourceMappingURL=index.js.map